# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, grpc_helpers_async, operations_v1
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
import grpc  # type: ignore
from grpc.experimental import aio  # type: ignore

from google.cloud.assuredworkloads_v1.types import assuredworkloads

from .base import DEFAULT_CLIENT_INFO, AssuredWorkloadsServiceTransport
from .grpc import AssuredWorkloadsServiceGrpcTransport


class AssuredWorkloadsServiceGrpcAsyncIOTransport(AssuredWorkloadsServiceTransport):
    """gRPC AsyncIO backend transport for AssuredWorkloadsService.

    Service to manage AssuredWorkloads.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "assuredworkloads.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "assuredworkloads.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: Optional[aio.Channel] = None,
        api_mtls_endpoint: Optional[str] = None,
        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[aio.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}
        self._operations_client: Optional[operations_v1.OperationsAsyncClient] = None

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def operations_client(self) -> operations_v1.OperationsAsyncClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Quick check: Only create a new client if we do not already have one.
        if self._operations_client is None:
            self._operations_client = operations_v1.OperationsAsyncClient(
                self.grpc_channel
            )

        # Return the client from cache.
        return self._operations_client

    @property
    def create_workload(
        self,
    ) -> Callable[
        [assuredworkloads.CreateWorkloadRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the create workload method over gRPC.

        Creates Assured Workload.

        Returns:
            Callable[[~.CreateWorkloadRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_workload" not in self._stubs:
            self._stubs["create_workload"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/CreateWorkload",
                request_serializer=assuredworkloads.CreateWorkloadRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_workload"]

    @property
    def update_workload(
        self,
    ) -> Callable[
        [assuredworkloads.UpdateWorkloadRequest], Awaitable[assuredworkloads.Workload]
    ]:
        r"""Return a callable for the update workload method over gRPC.

        Updates an existing workload. Currently allows updating of
        workload display_name and labels. For force updates don't set
        etag field in the Workload. Only one update operation per
        workload can be in progress.

        Returns:
            Callable[[~.UpdateWorkloadRequest],
                    Awaitable[~.Workload]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_workload" not in self._stubs:
            self._stubs["update_workload"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/UpdateWorkload",
                request_serializer=assuredworkloads.UpdateWorkloadRequest.serialize,
                response_deserializer=assuredworkloads.Workload.deserialize,
            )
        return self._stubs["update_workload"]

    @property
    def restrict_allowed_resources(
        self,
    ) -> Callable[
        [assuredworkloads.RestrictAllowedResourcesRequest],
        Awaitable[assuredworkloads.RestrictAllowedResourcesResponse],
    ]:
        r"""Return a callable for the restrict allowed resources method over gRPC.

        Restrict the list of resources allowed in the
        Workload environment. The current list of allowed
        products can be found at
        https://cloud.google.com/assured-workloads/docs/supported-products
        In addition to assuredworkloads.workload.update
        permission, the user should also have
        orgpolicy.policy.set permission on the folder resource
        to use this functionality.

        Returns:
            Callable[[~.RestrictAllowedResourcesRequest],
                    Awaitable[~.RestrictAllowedResourcesResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "restrict_allowed_resources" not in self._stubs:
            self._stubs["restrict_allowed_resources"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/RestrictAllowedResources",
                request_serializer=assuredworkloads.RestrictAllowedResourcesRequest.serialize,
                response_deserializer=assuredworkloads.RestrictAllowedResourcesResponse.deserialize,
            )
        return self._stubs["restrict_allowed_resources"]

    @property
    def delete_workload(
        self,
    ) -> Callable[[assuredworkloads.DeleteWorkloadRequest], Awaitable[empty_pb2.Empty]]:
        r"""Return a callable for the delete workload method over gRPC.

        Deletes the workload. Make sure that workload's direct children
        are already in a deleted state, otherwise the request will fail
        with a FAILED_PRECONDITION error.

        Returns:
            Callable[[~.DeleteWorkloadRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_workload" not in self._stubs:
            self._stubs["delete_workload"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/DeleteWorkload",
                request_serializer=assuredworkloads.DeleteWorkloadRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_workload"]

    @property
    def get_workload(
        self,
    ) -> Callable[
        [assuredworkloads.GetWorkloadRequest], Awaitable[assuredworkloads.Workload]
    ]:
        r"""Return a callable for the get workload method over gRPC.

        Gets Assured Workload associated with a CRM Node

        Returns:
            Callable[[~.GetWorkloadRequest],
                    Awaitable[~.Workload]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_workload" not in self._stubs:
            self._stubs["get_workload"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/GetWorkload",
                request_serializer=assuredworkloads.GetWorkloadRequest.serialize,
                response_deserializer=assuredworkloads.Workload.deserialize,
            )
        return self._stubs["get_workload"]

    @property
    def list_workloads(
        self,
    ) -> Callable[
        [assuredworkloads.ListWorkloadsRequest],
        Awaitable[assuredworkloads.ListWorkloadsResponse],
    ]:
        r"""Return a callable for the list workloads method over gRPC.

        Lists Assured Workloads under a CRM Node.

        Returns:
            Callable[[~.ListWorkloadsRequest],
                    Awaitable[~.ListWorkloadsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_workloads" not in self._stubs:
            self._stubs["list_workloads"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/ListWorkloads",
                request_serializer=assuredworkloads.ListWorkloadsRequest.serialize,
                response_deserializer=assuredworkloads.ListWorkloadsResponse.deserialize,
            )
        return self._stubs["list_workloads"]

    @property
    def list_violations(
        self,
    ) -> Callable[
        [assuredworkloads.ListViolationsRequest],
        Awaitable[assuredworkloads.ListViolationsResponse],
    ]:
        r"""Return a callable for the list violations method over gRPC.

        Lists the Violations in the AssuredWorkload Environment. Callers
        may also choose to read across multiple Workloads as per
        `AIP-159 <https://google.aip.dev/159>`__ by using '-' (the
        hyphen or dash character) as a wildcard character instead of
        workload-id in the parent. Format
        ``organizations/{org_id}/locations/{location}/workloads/-``

        Returns:
            Callable[[~.ListViolationsRequest],
                    Awaitable[~.ListViolationsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_violations" not in self._stubs:
            self._stubs["list_violations"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/ListViolations",
                request_serializer=assuredworkloads.ListViolationsRequest.serialize,
                response_deserializer=assuredworkloads.ListViolationsResponse.deserialize,
            )
        return self._stubs["list_violations"]

    @property
    def get_violation(
        self,
    ) -> Callable[
        [assuredworkloads.GetViolationRequest], Awaitable[assuredworkloads.Violation]
    ]:
        r"""Return a callable for the get violation method over gRPC.

        Retrieves Assured Workload Violation based on ID.

        Returns:
            Callable[[~.GetViolationRequest],
                    Awaitable[~.Violation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_violation" not in self._stubs:
            self._stubs["get_violation"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/GetViolation",
                request_serializer=assuredworkloads.GetViolationRequest.serialize,
                response_deserializer=assuredworkloads.Violation.deserialize,
            )
        return self._stubs["get_violation"]

    @property
    def acknowledge_violation(
        self,
    ) -> Callable[
        [assuredworkloads.AcknowledgeViolationRequest],
        Awaitable[assuredworkloads.AcknowledgeViolationResponse],
    ]:
        r"""Return a callable for the acknowledge violation method over gRPC.

        Acknowledges an existing violation. By acknowledging
        a violation, users acknowledge the existence of a
        compliance violation in their workload and decide to
        ignore it due to a valid business justification.
        Acknowledgement is a permanent operation and it cannot
        be reverted.

        Returns:
            Callable[[~.AcknowledgeViolationRequest],
                    Awaitable[~.AcknowledgeViolationResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "acknowledge_violation" not in self._stubs:
            self._stubs["acknowledge_violation"] = self.grpc_channel.unary_unary(
                "/google.cloud.assuredworkloads.v1.AssuredWorkloadsService/AcknowledgeViolation",
                request_serializer=assuredworkloads.AcknowledgeViolationRequest.serialize,
                response_deserializer=assuredworkloads.AcknowledgeViolationResponse.deserialize,
            )
        return self._stubs["acknowledge_violation"]

    def close(self):
        return self.grpc_channel.close()

    @property
    def get_operation(
        self,
    ) -> Callable[[operations_pb2.GetOperationRequest], operations_pb2.Operation]:
        r"""Return a callable for the get_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_operation" not in self._stubs:
            self._stubs["get_operation"] = self.grpc_channel.unary_unary(
                "/google.longrunning.Operations/GetOperation",
                request_serializer=operations_pb2.GetOperationRequest.SerializeToString,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["get_operation"]

    @property
    def list_operations(
        self,
    ) -> Callable[
        [operations_pb2.ListOperationsRequest], operations_pb2.ListOperationsResponse
    ]:
        r"""Return a callable for the list_operations method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_operations" not in self._stubs:
            self._stubs["list_operations"] = self.grpc_channel.unary_unary(
                "/google.longrunning.Operations/ListOperations",
                request_serializer=operations_pb2.ListOperationsRequest.SerializeToString,
                response_deserializer=operations_pb2.ListOperationsResponse.FromString,
            )
        return self._stubs["list_operations"]


__all__ = ("AssuredWorkloadsServiceGrpcAsyncIOTransport",)
